@@ -269,7 +269,7 @@ size_t findCovariateDimension(const std::string& fn, int column) {
   }
         
   stringNaturalTokenize(line, "\t ", &fd);
-  if (column >= fd.size() ) {
+  if (column >= (int)fd.size() ) {
     return ret;
   }
   // REprintf("all_cov = %s\n", line.c_str());
@@ -307,6 +307,8 @@ size_t findCovariateDimension(const std::string& fn, int column) {
 #define RET_COV_ZZ_INDEX 18
 #define RET_HWE_CASE_INDEX 19
 #define RET_HWE_CTRL_INDEX 20
+#define RET_AF_CASE_INDEX 21
+#define RET_AF_CTRL_INDEX 22
 
 SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
                          const OrderedMap< std::string, std::string>& geneRange) {
@@ -369,7 +371,7 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
   PROTECT(geneNames = allocVector(STRSXP, nGene));
   numAllocated ++;
   {
-    for (int i = 0; i < geneLocationMap.size(); ++i) {
+    for (size_t i = 0; i != geneLocationMap.size(); ++i) {
     // for ( GeneLocationMap::iterator iter = geneLocationMap.begin();
     //       iter != geneLocationMap.end() ; ++iter){
       // Rprintf("assign gene name: %s\n", iter->first.c_str());
@@ -403,12 +405,14 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
   names.push_back("covZZ");
   names.push_back("hweCase");
   names.push_back("hweCtrl");
+  names.push_back("afCase");
+  names.push_back("afCtrl");
   
   // REprintf("create zDims\n");
   std::vector<size_t> zDims(FLAG_covFile.size(), 0);
   // GeneLocationMap::iterator iter = geneLocationMap.begin();
   for ( int i = 0;
-        i < geneLocationMap.size();
+        i < (int) geneLocationMap.size();
         ++i) {
     // REprintf("i = %d\n", i);
     // REprintf("names.size() = %zu\n", names.size());
@@ -417,7 +421,8 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
     numAllocated += createList(names.size(), &s); // a list with 10 elements: ref, alt, n, maf, stat, direction, p, cov, pos, anno
     numAllocated += setListNames(names, &s);
 
-    SEXP ref, alt, n, af, ac, callRate, hwe, nref, nhet, nalt, ustat, vstat, effect, p, cov, pos, anno, covXZ, covZZ, hweCase, hweCtrl;
+    SEXP ref, alt, n, af, ac, callRate, hwe, nref, nhet, nalt, ustat, vstat, effect, p, cov, pos, anno,
+        covXZ, covZZ, hweCase, hweCtrl, afCase, afCtrl;
     numAllocated += createList(nStudy, &ref);
     numAllocated += createList(nStudy, &alt);
     numAllocated += createList(nStudy, &n);
@@ -439,6 +444,8 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
     numAllocated += createList(nStudy, &covZZ);
     numAllocated += createList(nStudy, &hweCase);
     numAllocated += createList(nStudy, &hweCtrl);
+    numAllocated += createList(nStudy, &afCase);
+    numAllocated += createList(nStudy, &afCtrl);
     
     int npos = geneLocationMap.valueAt(i).size();
     // std::vector<size_t> zDims(FLAG_covFile.size(), 0);
@@ -556,6 +563,16 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
       numAllocated += createDoubleArray(npos, &t);
       initDoubleArray(t);
       SET_VECTOR_ELT(hweCtrl, j, t);
+
+      //allocate memory for afCase, afCtrl
+      numAllocated += createDoubleArray(npos, &t);
+      initDoubleArray(t);
+      SET_VECTOR_ELT(afCase, j, t);
+
+      numAllocated += createDoubleArray(npos, &t);
+      initDoubleArray(t);
+      SET_VECTOR_ELT(afCtrl, j, t);
+      
     } // end looping study
     numAllocated += createStringArray(npos, &pos);
     initStringArray(pos);
@@ -584,6 +601,8 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
     SET_VECTOR_ELT(s, RET_COV_ZZ_INDEX, covZZ);
     SET_VECTOR_ELT(s, RET_HWE_CASE_INDEX, hweCase);
     SET_VECTOR_ELT(s, RET_HWE_CTRL_INDEX, hweCtrl);
+    SET_VECTOR_ELT(s, RET_AF_CASE_INDEX, afCase);
+    SET_VECTOR_ELT(s, RET_AF_CTRL_INDEX, afCtrl);
         
     SET_VECTOR_ELT(ret, i, s);
   };
@@ -679,7 +698,7 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
       double tempDouble;
       while( tr.readLine(&line) ){
         stringNaturalTokenize(line, " \t", &fd);
-        if (fd.size() <= PVAL_FILE_MIN_COLUMN_NUM) continue;
+        if ((int)fd.size() <= PVAL_FILE_MIN_COLUMN_NUM) continue;
         p = fd[PVAL_FILE_CHROM_COL];
         p += ':';
         p += fd[PVAL_FILE_POS_COL];
@@ -716,10 +735,30 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
           INTEGER(s)[idx] = tempInt;
         }
 
-        if ( str2double(fd[PVAL_FILE_AF_COL], &tempDouble) ) {
-          v = VECTOR_ELT(u, RET_AF_INDEX);
-          s = VECTOR_ELT(v, study); // af
-          REAL(s)[idx] = tempDouble;
+        // af field may have one pvalue or three pvalue (all:case:control)
+        std::vector<std::string> afPvalues;
+        // REprintf("af = %s\n", fd[PVAL_FILE_AF_COL].c_str());
+        stringTokenize(fd[PVAL_FILE_AF_COL], ":", &afPvalues);
+        if (!afPvalues.empty()) {
+          if ( str2double(afPvalues[0], &tempDouble) ) {
+            v = VECTOR_ELT(u, RET_AF_INDEX);
+            s = VECTOR_ELT(v, study); // af
+            REAL(s)[idx] = tempDouble;
+          }
+          if (afPvalues.size() == 3) { // af_all:af_case:af_ctrl
+            if ( str2double(afPvalues[1], &tempDouble) ) {
+              v = VECTOR_ELT(u, RET_AF_CASE_INDEX);
+              s = VECTOR_ELT(v, study); // af
+              REAL(s)[idx] = tempDouble;
+            }
+            if ( str2double(afPvalues[2], &tempDouble) ) {
+              v = VECTOR_ELT(u, RET_AF_CTRL_INDEX);
+              s = VECTOR_ELT(v, study); // af
+              REAL(s)[idx] = tempDouble;
+            }
+          }
+        }  else {
+          REprintf("AF column has incorrect value [ %s ]", fd[PVAL_FILE_AF_COL].c_str());
         }
 
         if ( str2int(fd[PVAL_FILE_AC_COL], &tempInt) ) {
@@ -817,7 +856,7 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
   // fill in position and annotation
   // iter = geneLocationMap.begin();
   for ( int i = 0;
-        i < geneLocationMap.size();
+        i < (int)geneLocationMap.size();
         ++i) {
     // iter != geneLocationMap.end() ; ++iter, ++i){
     const std::map<std::string, int>& loc2idx = geneLocationMap.valueAt(i);
@@ -908,7 +947,7 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
           // REprintf("line = %s\n", line.c_str());
 
           stringNaturalTokenize(line, " \t", &fd);
-          if (fd.size() <= COV_FILE_MIN_COLUMN_NUM) continue;
+          if ((int)fd.size() <= COV_FILE_MIN_COLUMN_NUM) continue;
           
           const std::string& chrom = fd[COV_FILE_CHROM_COL];
           // REprintf("pos: %s\n", fd[COV_FILE_COV_COL].c_str());
@@ -963,10 +1002,10 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
           stringNaturalTokenize(covArray[1], ',', &covXZ);
           // REprintf("covXZ = %s\n", covArray[1].c_str());
           int zDim = zDims[study];
-          if (covXZ.size() != zDim) {
+          if ((int)covXZ.size() != zDim) {
             REprintf("CovXZ has wrong dimension (Covariance column = [ %s ])\n", fd[COV_FILE_COV_COL].c_str() );
           }
-          for (size_t j = 0; j < zDim; ++j) {
+          for (int j = 0; j < zDim; ++j) {
             pj = chrom + ":" + pos[0];
             // REprintf("pj = %s, covXZ[j] = %s\n", pj.c_str(), covXZ[j].c_str());
             if (location2idx.count(pj) == 0) {
@@ -982,7 +1021,7 @@ SEXP impl_rvMetaReadData(SEXP arg_pvalFile, SEXP arg_covFile,
           v = VECTOR_ELT(u, RET_COV_ZZ_INDEX);  // cov is the 6th element in the list
           s = VECTOR_ELT(v, study);
           stringNaturalTokenize(covArray[2], ',', &covZZ);
-          if (zDim * (zDim + 1) / 2 != covZZ.size()) {
+          if (zDim * (zDim + 1) / 2 != (int)covZZ.size()) {
             REprintf("CovXZ has wrong dimension (Covariance column = [ %s ])\n", fd[COV_FILE_COV_COL].c_str() );
           }
           // REprintf("covZZ = %s\n", covArray[2].c_str());
@@ -1128,7 +1167,7 @@ SEXP impl_readCovByRange(SEXP arg_covFile, SEXP arg_range) {
       // Rprintf("%s\n", s);
       line = s;
       stringTokenize(line, "\t", &fd);
-      if (fd.size() <= COV_FILE_MIN_COLUMN_NUM) continue;
+      if ((int)fd.size() <= COV_FILE_MIN_COLUMN_NUM) continue;
       // if (fd[COV_FILE_NUM_MARKER_COL] == "1") continue; // only self covariance
       lineNo ++;
 
@@ -1401,7 +1440,7 @@ SEXP impl_readScoreByRange(SEXP arg_scoreFile, SEXP arg_range) {
   while (tr.readLine(&line)) {
     // Rprintf("read a line: %s\n", line.c_str());
     stringNaturalTokenize(line, "\t ", &fd);
-    if (fd.size() <= PVAL_FILE_MIN_COLUMN_NUM) continue;
+    if ((int)fd.size() <= PVAL_FILE_MIN_COLUMN_NUM) continue;
     int pos = atoi(fd[PVAL_FILE_POS_COL]);
 
     // check consistent column number
